// Prevents additional console window on Windows in release, DO NOT REMOVE!!
#![cfg_attr(not(debug_assertions), windows_subsystem = "windows")]
use id3::{Tag, TagLike};
use serde::Serialize;
use std::collections::HashSet;
use walkdir::WalkDir;

#[derive(Debug, Serialize, PartialEq, Eq, Hash)]
pub struct Album {
    pub name: String,
    pub artist: String,
    pub year: i32,
    pub lossless: bool,
}

#[tauri::command]
async fn get_local_albums(dirpath: String) -> Result<Vec<Album>, String> {
    let mut result = HashSet::new();
    for entry in WalkDir::new(dirpath) {
        let entry = entry.map_err(|e| e.to_string())?;
        println!("{}", entry.path().display());
        if entry.file_type().is_file() {
            let path = entry.path();
            if let Ok(tag) = Tag::read_from_path(path) {
                let album = Album {
                    name: tag.album().unwrap_or("").to_string(),
                    artist: tag.artist().unwrap_or("").to_string(),
                    year: tag.year().unwrap_or(0),
                    lossless: entry
                        .path()
                        .extension()
                        .map(|ext| ext == "flac" || ext == "wav")
                        .unwrap_or(false),
                };
                result.insert(album);
            }
        }
    }
    Ok(result.into_iter().collect())
}

fn main() {
    // https://github.com/tauri-apps/tauri/issues/5143#issuecomment-1311815517
    std::env::set_var("WEBKIT_DISABLE_COMPOSITING_MODE", "1");

    tauri::Builder::default()
        .invoke_handler(tauri::generate_handler![get_local_albums])
        .run(tauri::generate_context!())
        .expect("error while running tauri application");
}
